/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ca.mcscert.jtet.expression;

import java.util.Arrays;
import java.util.Collection;

/**
 *
 * @author matthew
 */
public enum BinaryOperation {

    Addition,
    Minus,
    Multiplication,
    Division,
    Exponentiation,

    GreaterThen,
    GreaterThenEqual,
    LessThen,
    LessThenEqual,
    Equals,
    NotEquals,

    Implies,
    BooleanAnd,
    BooleanOr;

    Collection<TypeMarker> getInputTypeForOutput(TypeMarker outputType) {
        // Boolean inputs only work with a boolean input!
        if (this == BooleanAnd || this == BooleanOr || this == Implies || this == GreaterThen || this == GreaterThenEqual ||
                this == LessThen || this == LessThenEqual || this == Equals || this == NotEquals) {
            if (!outputType.equals(BooleanType.Type)) {
                throw new IllegalArgumentException("Boolean operators and comparisons require a boolean output type!");
            }
        }
        if (outputType instanceof EnumerationType) {
            if (this != Equals && this != NotEquals) {
                throw new IllegalArgumentException("Enumeration types can only be checked for equality!");
            }
        }

        // Comparisions produce boolean outputs!
        switch(this) {
            case GreaterThen:
            case GreaterThenEqual:
            case LessThen:
            case LessThenEqual:
                return Arrays.asList(new TypeMarker[]{RealType.Type, FixedPointType.TypeMarker});
            case Equals:
            case NotEquals:
                return Arrays.asList(new TypeMarker[]{BooleanType.Type, RealType.Type, FixedPointType.TypeMarker, EnumerationType.TypeMarker});
        }
        return Arrays.asList(new TypeMarker[]{outputType});
    }

    public Type getOutputForInput(Type inputType) {
        // Boolean inputs only work with a boolean input!
        if (this == BooleanAnd || this == BooleanOr || this == Implies) {
            if (!inputType.equals(BooleanType.Type)) {
                throw new IllegalArgumentException("Boolean operators require a boolean input type!");
            }
        }

        // Comparisions produce boolean outputs!
        switch(this) {
            case GreaterThen:
            case GreaterThenEqual:
            case LessThen:
            case LessThenEqual:
            case Equals:
            case NotEquals:
                return BooleanType.Type;
        }
        return inputType;
    }
}
