/*
 * Copyright (C) 2014 Matthew Dawson <matthew@mjdsystems.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package ca.mcscert.jtet.salgenerator;

import ca.mcscert.jtet.expression.BooleanType;
import ca.mcscert.jtet.expression.Expression;
import ca.mcscert.jtet.expression.Variable;
import ca.mcscert.jtet.tabularexpression.HierarchicalGridDepthFirstCheckerGenerator;

import java.util.Map;

/**
 * Created by matthew on 6/3/14.
 */
public class HierarchicalGridSALGenerator implements HierarchicalGridDepthFirstCheckerGenerator {

    public HierarchicalGridSALGenerator(Variable outputVariable)
    {
        this.m_outputVariable = outputVariable;
    }

    @Override
    public void descendIntoGridFromCell(Expression inputCellExpression)
    {
        outputCellPrelude(inputCellExpression);
        m_firstCell = true;
    }

    @Override
    public void ascendFromGrid()
    {
        handleEndIf();
        result.append(" ");
        m_firstCell = false;
    }

    private void outputCellPrelude(Expression inputCellExpression)
    {
        m_currentlyRunning = true;
        if (m_firstCell) {
            result.append("IF ");
        } else {
            result.append("ELSIF ");
        }
        m_firstCell = false;
        result.append(inputCellExpression.getCheckerOutput(m_generator, BooleanType.Type));
        result.append(" THEN ");
    }

    @Override
    public void handleLeafCell(Expression inputCellExpression, Map<Variable, Expression> outputCellsExpression)
    {
        if (m_firstOutputCellExpression == null) {
            m_firstOutputCellExpression = outputCellsExpression.get(m_outputVariable);
        }
        outputCellPrelude(inputCellExpression);
        result.append(getOutputExpressionAsSALCode(outputCellsExpression.get(m_outputVariable)));
        result.append(" ");
    }

    @Override
    public void handleEdgeCell(Expression inputCellExpression)
    {
    }

    @Override
    public String getFinalString()
    {
        if (m_currentlyRunning && result.length() != 0) {
            handleEndIf();
            m_currentlyRunning = false;
        }
        return result.toString();
    }

    private void handleEndIf() {
        result.append("ELSE ")
                .append(getOutputExpressionAsSALCode(m_firstOutputCellExpression))
                .append(" ENDIF");
    }

    private String getOutputExpressionAsSALCode(Expression expression) {
        return expression.getCheckerOutput(m_generator, m_outputVariable.type());
    }

    final private StringBuilder result = new StringBuilder();

    private boolean m_currentlyRunning = false;
    private boolean m_firstCell = true;
    private final Variable m_outputVariable;
    private Expression m_firstOutputCellExpression;

    final private SALExpressionGenerator m_generator = new SALExpressionGenerator();
}
