/*
 * Copyright (C) 2013 Matthew Dawson <matthew@mjdsystems.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package ca.mcscert.jtet.smtlibchecker;

import ca.mcscert.jtet.expression.*;

import java.util.Collection;

/**
 *
 * @author Matthew Dawson <matthew@mjdsystems.ca>
 */
public class SMTLIBVariablesDeclarationGenerator implements VariablesDeclarationGenerator {
    public final static SMTLIBVariablesDeclarationGenerator Generator = new SMTLIBVariablesDeclarationGenerator();

    @Override
    public String GenerateVariablesDeclaration(Collection<Variable> variables) {
        StringBuilder ret = new StringBuilder();

        //Generate all the variables.
        for (Variable var : variables) {
            ret.append("(declare-fun ")
               .append(var.outputName())
               .append(" () ")
               .append(GenerateType(var.type()))
               .append(")\n");

            // If variable is an enumeration, make sure that it is limited to only enumeration values.
            if (var.type() instanceof EnumerationType) {
                ret.append("(assert (is")
                        .append(((EnumerationType) var.type()).name())
                        .append(" ")
                        .append(var.outputName())
                        .append("))\n");
            }
        }

        // Then generate the variable predicates.  This ensures any dependent variables are declared.
        for(Variable var : variables) {
            if (var.subtypePredicate() != null) {
                ret.append("(assert ")
                        .append(var.subtypePredicate().getCheckerOutput(SMTLIBExpressionGenerator.Generator, BooleanType.Type))
                        .append(")\n");
            }
        }
        return ret.toString();
    }

    /**@deprecated
     * it is only public for unit tests, and will be made private eventually
     */
    public static String GenerateType(Type type) {
        if (type instanceof RealType) {
            return "Real";
        } else if (type instanceof BooleanType) {
            return "Bool";
        } else if (type instanceof EnumerationType) {
            return ((EnumerationType) type).name();
        } else {
            throw new IllegalArgumentException("SMT-LIB Doesn't handle this variable type (Class: " + type.getClass().getName() + ")");
        }
    }
}
