/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ca.mcscert.jtet.expression;

import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.*;

import ca.mcscert.jtet.cvc3generator.CVC3VariablesDeclarationGenerator;
import ca.mcscert.jtet.smtlibchecker.SMTLIBVariablesDeclarationGenerator;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.JUnit4;

import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author matthew
 */
@RunWith(JUnit4.class)
public class VariableTest {
    @Test
    public void exerciseVariable() {
        String name = "Var";
        Type type = new RealType();

        Variable var = new Variable(name, type);

        assertSame(name, var.name());
        assertSame(type, var.type());
        assertEquals(name, var.getCheckerOutput(null, RealType.Type));
    }

    @Test
    public void testSubtypePredicate() {
        // Construct a basic variable.  Assume it is correct due to the above test.
        Variable var = new Variable("var", new RealType());

        // Next create a subtype predicate.  For now, make it work over positive reals (excluding 0)
        Expression predicate = new ExpressionBinaryOperation(var, BinaryOperation.GreaterThen, new Literal("0"));
        var.setSubtypePredicate(predicate);

        // Verify the predicate took.
        assertThat(var.subtypePredicate(), is(predicate));
    }

    @Test
    public void testOutputNameChangeInGenerator() {
        Variable var = new Variable("var", RealType.Type);

        assertThat(var.name(), is("var"));
        assertThat(var.toString(), is("var"));
        assertThat(var.outputName(), is("var"));
        assertThat(var.getCheckerOutput(null, RealType.Type), is("var"));

        var.setOutputName("new_name");

        assertThat(var.name(), is("var"));
        assertThat(var.toString(), is("var"));
        assertThat(var.outputName(), is("new_name"));
        assertThat(var.getCheckerOutput(null, RealType.Type), is("new_name"));
    }

    // Verify handling of invalid type to getCheckerOutput
    @Test(expected=IllegalArgumentException.class)
    public void testInvalidOutputType() {
        String name = "Var";
        Type type = BooleanType.Type;

        Variable var = new Variable(name, type);

        var.getCheckerOutput(null, RealType.Type); //Returns a boolean, so will fail.
    }
}
