/*
 * Copyright (C) 2014 Matthew Dawson <matthew@mjdsystems.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package ca.mcscert.jtet.expression.test;

import ca.mcscert.jtet.expression.EnumerationType;
import ca.mcscert.jtet.expression.PartialVariableCollection;
import ca.mcscert.jtet.expression.RealType;
import ca.mcscert.jtet.expression.Variable;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.JUnit4;

import java.util.HashMap;
import java.util.Map;

import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.CoreMatchers.notNullValue;
import static org.junit.Assert.assertThat;

@RunWith(JUnit4.class)
public class PartialVariableCollectionTest {
    @Test
    public void singleArgumentConstructor() {
        Variable var1 = new Variable("test1", RealType.Type);
        Variable var2 = new Variable("test2", RealType.Type);

        PartialVariableCollection vars = new PartialVariableCollection(var1, var2);

        assertThat(vars.m_variables.size(), is(2));
        assertThat(vars.m_variables.get("test1"), is(var1));
        assertThat(vars.m_variables.get("test2"), is(var2));
        assertThat(vars.m_enumerationTypes, is(notNullValue()));
        assertThat(vars.m_enumerationTypes.size(), is(0));
    }

    @Test
    public void singleArgumentConstructorEnumerations() {
        final EnumerationType enumerationType = new EnumerationType("testtype");
        enumerationType.enumerationValues().add("testconst ");
        Variable var = new Variable("test", enumerationType);

        PartialVariableCollection vars = new PartialVariableCollection(var);

        assertThat(vars.m_variables.size(), is(1));
        assertThat(vars.m_variables.get("test"), is(var));
        assertThat(vars.m_enumerationTypes, is(notNullValue()));
        assertThat(vars.m_enumerationTypes.size(), is(1));
        assertThat(vars.m_enumerationTypes.contains(enumerationType), is(true));
    }
}
